import os
import svgwrite
from svgwrite import cm,mm,rgb
import pathlib
running_height = 0
max_width = 0

def main():
    pass

def set_width(value):
    global max_width
    max_width = value

RECT_HEIGHT=40
SPACING = 20
# input:
# folder: path to the folder inside svgs folder
# transcript: iterable with user_exons,domains,db_exons
def visualize_transcript(folder,transcript):
    pathlib.Path('svgs/{}'.format(folder)).mkdir(parents=True, exist_ok=True)

    user_exons = transcript[0]
    domains = transcript[1]
    db_exons = transcript[2]
    draw_transcript(folder,user_exons,domains,db_exons)

def draw_transcript(folder,user_exons,domains, db_exons):
    running_height = 0
    if not user_exons:
        print('no user exons')
        return
    #transcript_start = int(user_exons[0]['start'])
    #transcript_end = int(user_exons[-1]['end'])
    rel_start = 1
    rel_end = int(user_exons[-1]['relative_end'])
    set_width(rel_end)
    gene_name = user_exons[0]['gene_id']
    transcript_id = user_exons[0]['transcript_id']
    # draw the transcript
    dwg = svgwrite.Drawing(filename='svgs/{}/{}.svg'.format(folder,transcript_id),size=('100%','100%'))
    dwg.defs.add(dwg.style('''
        text
        { visibility: hidden;   pointer-events: none;}
        #shown
        { visibility: visible; }
        rect:hover
        { opacity: 0.5; }
        rect:hover ~ text
        { visibility: visible; }
        '''))
    draw_rect(dwg,(1,0),rel_end-1,RECT_HEIGHT,text=transcript_id, is_trans=True)
    #dwg.add(dwg.text(text=str(rel_start),insert=(rel_start,RECT_HEIGHT+1+running_height),font_size=20))
    #dwg.add(dwg.text(text=str(rel_end),insert=(rel_end,RECT_HEIGHT+1+running_height),font_size=20))
    running_height += RECT_HEIGHT + SPACING
    draw_domains(dwg,domains,running_height)
    draw_user_exons(dwg,user_exons,running_height)
    draw_db_exons(dwg,db_exons,running_height)
    dwg.save()



def normalize(value):
    if max_width == 0:
        print("failed to normalize")
        return value
    return value/max_width * 400

def draw_rect(dwg,position,width,height,fill_color='red',stroke_color='black',text=None,is_trans=False,tooltip_text = None):
    start_txt = str(position[0])
    end_txt = str(position[0] + width)
    width = normalize(width)
    position = normalize(position[0]),position[1]
    g = dwg.add(dwg.g(opacity=0.8))

    rect = g.add(dwg.rect(id="myrect",insert=position,size=(width,height),fill=fill_color,stroke=stroke_color))
    if tooltip_text:
        rect.set_desc(title=tooltip_text,desc=tooltip_text)
    if text:
        text_pos = position[0] + width/2,position[1] + height/3
        g.add(dwg.text(text=text,id="shown",insert=text_pos,font_size=20,style="text-anchor: middle; dominant-baseline: hanging;"))

    dwg.add(dwg.line(start=(position[0],position[1]+height-10),end=(position[0],position[1]+height+10),stroke='black',stroke_width=2))
    dwg.add(dwg.line(start=(position[0]+width,position[1]+height-10),end=(position[0]+width,position[1]+height+10),stroke='black',stroke_width=2))

    g.add(dwg.text(text=start_txt,insert=(position[0],position[1]+RECT_HEIGHT+running_height),font_size=14))
    mod = 0
    if is_trans:
        mod = RECT_HEIGHT
    g.add(dwg.text(text=end_txt,insert=(position[0]+width,position[1]+running_height+mod),font_size=14))
    return g
    #print (start_txt,' ',end_txt)

def draw_domains(dwg,domains,running_height):
    if not domains:
        dwg.add(dwg.text("no domains",(0,50)))
        return
    #print(dwg.filename)
    for domain in domains:
        draw_domain(dwg,domain)
    running_height += SPACING*3

def draw_domain(dwg,domain):
    start = int(domain['start'])*3 - 2
    end = int(domain['end']) * 3
    domain['start'] = start
    domain['end'] = end
    position = (start,running_height+2*SPACING)
    height = RECT_HEIGHT/2
    width = end-start
    g = draw_rect(dwg,position,width,height,'blue','black',tooltip_text=str(domain))
    #domain_data_pos = (0, position[1]+running_height+SPACING*4)
    #g.add(dwg.text(text=str(domain),insert=domain_data_pos,font_size=14))

def draw_user_exons(dwg,exons,running_height):
    exons_g = dwg.add(dwg.g(id='exons',font_size=14))
    if not exons:
        exons_g.add(dwg.text("no exons",(0,100)))
        return
    for exon in exons:
        draw_exon(dwg,exon,running_height)
    running_height += SPACING*3

def draw_db_exons(dwg,exons,running_height):
    exons_g = dwg.add(dwg.g(id='db_exons',font_size=14))
    if not exons:
        exons_g.add(dwg.text("no db_exons",(0,100)))
        return
    for exon in exons:
        draw_exon(dwg,exon,running_height+SPACING,color='yellow')
    running_height += SPACING*3



def draw_exon(dwg,exon,running_height,color=None):
    if not color:
        color = 'green'
    start = int(exon['relative_start'])
    end = int(exon['relative_end'])
    width = end-start
    position = start,running_height
    height = RECT_HEIGHT/2
    g = draw_rect(dwg,position ,width,height,color,'black',tooltip_text=str(exon))
    #exon_data_pos = (0, position[1]+SPACING*4)
    #g.add(dwg.text(text=str(exon),insert=exon_data_pos,font_size=14))



if __name__ == '__main__':
    main()




import os
import json
import progressbar
import sys
import domains_to_exons
import sqlite3 as lite
import conf

transcript_db = 'db/transcript_data.db'
domains_db = 'db/domains.db'

# RELATIVE CALCULATIONS ARE PROBABLY WRONG
# MIGHT NEED TO GO BY TOTAL POSTITIONS OR
# FIND A WAY TO HAVE CORRECT RELATIVE POSTITIONS
def parse_gtf(file_path):
    with open(file_path) as f:
        lines = f.readlines()    # dictionary of exons by transcript_id
    transcripts = {}
    transcript_id_prev = ''
    gene_id_prev = ''
    relative_end = 0
    exons = []
    for line in lines:
        splitted = line.split("\t")
        # check if feature is exon

        if splitted[2] == 'exon':
            exon = {}
            exon['gene_id'] = splitted[8].split('"')[1]
            exon['transcript_id'] = splitted[8].split('"')[3]
            exon['cds'] = {}
            exon['cds']['start'] = int(splitted[3])
            exon['cds']['end'] = int(splitted[4])
            exon['start'] = exon['cds']['start']
            exon['exon_starts'] = exon['cds']['start']
            exon['end'] = exon['cds']['end']
            exon['exon_ends'] = exon['cds']['end']
            exon['index'] = int(splitted[8].split('"')[5])
            # add one to the length
            exon['cds']['length'] = abs(exon['cds']['end'] - exon['cds']['start']) + 1
            # increment relative start location
            if exon['transcript_id'] == transcript_id_prev:
                relative_start = relative_end + 1
                #relative_start = relative_end + 1 + abs(last_exon['end'] - exon['start'])
            # reset relative start location
            else:
                exons = []
                relative_start = 1

            relative_end = relative_start + exon['cds']['length']
            exon['relative_start'] = relative_start
            exon['relative_end'] = relative_end
            last_exon = exon
            transcript_id_prev = exon['transcript_id']
            gene_id_prev = exon['gene_id']
            exons.append(exon)
            transcripts[exon['transcript_id']] = exons
        # maybe remove first element of transcripts
    # maybe not
    return transcripts


# compare the domains of user exon and db exon
def compare_domains(u_exon,exon):
    if u_exon['start'] == exon['start'] and u_exon['end'] == exon['end']:
                return 'identical'
    if u_exon['domains_states'] == exon['domains_states']:
        if u_exon['domains'] == exon['domains']:
                return 'domains match'
        return 'domains_states match'
    return 'different'

# check how u_exon compares to the database exons
def compare_exons(u_exon,exons):
    # TODO - decide what should be the compare function
    u_exon['relations'] = []
    for exon in exons:
        u_exon['relations'].append(compare_domains(u_exon,exon))


# returns the domains associated with a given exon index and transcript id 
# input:
# exon: dict with transcript_id and index
# specie: string of the specie (must be one from conf.py)
def get_exon_domains(exon,specie):
    with lite.connect(conf.databases[specie]) as con:
        cursor = con.cursor()
        cursor.execute("SELECT domains_list from domains WHERE transcript_id = ? AND exon_index = ?",(exon['transcript_id'],exon['index']))
        result = cursor.fetchone()
    if not result:
       return None
    doms = result[0].split(',')
    return doms

# parse the given exons json string
# input:
# exons: list of dictionaries that contains json dump of domains and domains_states
# output:
# exons: list of dictionaris that contains domains and domains_states
def parse_exon_domains_to_dict(exons):
    for exon in exons:
        domains_string = exon['domains']#.replace("'",'"')
        domains_states_string = exon['domains_states']#.replace("'",'"')
        try:
            exon['domains'] = json.loads(domains_string)
            exon['domains_states'] = json.loads(domains_states_string)
        except :
            print("failed to load domains")
            print('dom: ',domains_string)
            print('dom_states: ',domains_states_string)
            sys.exit(2)
    return exons

# call when exons list need to load domains info from domains table
# input:
# exons: list of dictionaries that will be populated with json strings from the database
# specie: string of the specie (must be one from conf.py)
# output:
# True if loaded anything
# None if didn't load anything
def load_exons_domains(exons,specie):
    transcript_id = exons[0]['transcript_id']
    indexes = set([str(exon['index']) for exon in exons])
    # conncet do domains db
    with lite.connect(conf.databases[specie]) as con:
        cursor = con.cursor()
        cursor.execute("SELECT exon_index,domains_states,domains_list FROM domains WHERE transcript_id = ?",(transcript_id,))
        result = cursor.fetchall()
    # remember to check for None
    if not result:
        #print ('result is none:',result)
        return None
    for value in result:
        if value:
            # pack the domains_states and domains_list
            exons[int(value[0])]['domains_states'] = value[1]
            exons[int(value[0])]['domains'] = value[2]
            #print (exons[int(value[0])])
    parse_exon_domains_to_dict(exons)
    return True


# TODO if comparison method is changed - update this method.
# usage: call to fill exon with domain data
# input:
# exon: dictionary of exon data
# domains: list of domains (dictionaries)
# output:
# the exon's key domains_states will be a list of states (index will reference the domain's index)
# possible values: 'contained','start','end','contained'.
def assign_domains_to_exon(exon, domains):
    domains_in_exon = []
    exon['domains_states'] = {}
    for domain in domains:
        # do same test as in domains_to_exon.py
        dom_start = int(domain['start']) * 3 -2
        dom_end = int(domain['end']) * 3
        dom_range = range(dom_start,dom_end+1)
        exon_range = set(range(exon['relative_start'],exon['relative_end']))
        intersection = exon_range.intersection(dom_range)
        dom_start_in_exon = False
        dom_end_in_exon = False
        if not intersection:
            continue
        if dom_start in intersection:
            dom_start_in_exon = True
        if dom_end in intersection:
            dom_end_in_exon = True
        dom_index_string = str(domain['index'])
        if dom_end_in_exon and dom_start_in_exon:
            exon['domains_states'][dom_index_string] = 'contained'
            domains_in_exon.append(domain)
            continue
        if dom_start_in_exon:
            exon['domains_states'][dom_index_string] = 'start'
            domains_in_exon.append(domain)
            continue
        if dom_end_in_exon:
            exon['domains_states'][dom_index_string] = 'end'
            domains_in_exon.append(domain)
            continue
        exon['domains_states'][dom_index_string] = 'contains'
        domains_in_exon.append(domain)
        continue
    exon['domains'] = domains_in_exon

# usage: call when need to know what domains an exon contains
# takes transcript_data of user gtf file (cut up to dict)
# need to have atleast transcirpt_id and exons value
# input:
# u_transcript_id:  id of the transcript
# u_exons: list of dictionaries
# specie: string of the specie (must be one from conf.py)
# output:
# u_exons: the exons of the transcript id
# domains: the domains of the transcript id
def assign_gtf_domains_to_exons(u_transcript_id, u_exons,specie):
    domains = domains_to_exons.get_domains(u_transcript_id,specie)
    #print ('possible domains: ',domains)
    # check for failure
    if not domains or not u_exons:
        return u_exons, domains
    # for each u_exon, check every domain
    for u_exon in u_exons:
        assign_domains_to_exon(u_exon,domains)
    # can now do compare to exons step

    exons = domains_to_exons.get_exons_by_transcript_id(u_transcript_id,specie)
    # make sure there are exons
    if not exons:
        return u_exons, domains
    # load the exons domains
    # first extract all the domains so they are not extracted everytime
    if not load_exons_domains(exons,specie):
        # no domains for exons at all
        return u_exons, domains
    #exons_domains = list(map(get_exon_domains,exons,specie))
    #print('exons_doms: {}'.format(exons_domains))
    #for u_exon in u_exons:
        # u_exons[domains] will be a set of strings
        #compare_exons(u_exon,exons)
    return u_exons, domains
                # TODO domains assignments should be done in contain maybe
                # TODO or depend on string result
                # skip empty exons(no domains inside them)
                #if not exon_domains:
                # TODO DONE: COMPARISON IN compare_exons function
                #continue
                #u_exon['domains'].update(exon_domains)
                #print(exon_domains)
                #print(u_exon['domains'])
    # exons now have domains data

# the interface of dochap.
# input:
# input_file: path to input file
# output_file: path to the output file
# specie: string of the specie (must be one from conf.py)
# output:
# output_file: the path to the output file
def interface(input_file,output_file,specie):
    print('parsing {}...'.format(input_file))
    transcripts = parse_gtf(input_file)
    print('assigning domains to exons...')
    bar = progressbar.AnimatedProgressBar(end=len(transcripts),width=10)
    for transcript_id,exons in transcripts.items():
        transcripts[transcript_id] = assign_gtf_domains_to_exons(transcript_id,exons,specie)
        bar+=1
        bar.show_progress()
    bar+=1
    bar.show_progress()
    to_write = [(name,data) for name,data in transcripts.items() if data]
    with open(output_file,'w') as f:
        f.write(json.dumps(transcripts))
    # stop here
    return output_file


# takes argv
# usage - will be printed upon calling the script
def main():
    if len(sys.argv) < 4:
        print('inteface.py <specie> <inputfile> <outputfile>')
        sys.exit(2)
    specie = sys.argv[1]
    input_file = sys.argv[2]
    output_file = sys.argv[3]
    print('parsing {}...'.format(input_file))
    transcripts = parse_gtf(input_file)
    print('assigning domains to exons...')
    bar = progressbar.AnimatedProgressBar(end=len(transcripts),width=10)
    for transcript_id,exons in transcripts.items():
        transcripts[transcript_id] = assign_gtf_domains_to_exons(transcript_id,exons,specie)
        bar+=1
        bar.show_progress()
    bar+=1
    bar.show_progress()
    to_write = [(name,data) for name,data in transcripts.items() if data]
    with open(output_file,'w') as f:
        f.write(json.dumps(transcripts))
    # stop here, writing json dump is easier then creating something else
    return
    '''
    with open(output_file,'w') as f:
        for name,exons in to_write:
            f.write('{}:\n'.format(name))
            if not exons:
                f.write('None\n')
                continue
            for e in exons:
                doms = str(e.get('domains','[]'))
                states = str(e.get('domains_states','{}'))
                loc = str((e.get('start',None),e.get('end',None)))
                rel_loc = str((e.get('relative_start',None),e.get('relative_end',None)))
                f.write('index: {} loc: {}, rel_loc: {} states:{} domains: {}\n'.format(e['index'],loc,rel_loc,states,doms))
    print('done')
    '''
if __name__ == '__main__':
    main()




import sys
'''
DOCHAP (DOmain CHAnge Predictor) takes the data file containing the domain information made in Matlab and a gtf file from an
experiment. The domain Data file name will go in the loadData function and the user experiment file name will go in the
parseGTF function.
As an output you will get a file containing a list of the user's transcripts. For each transcript there will be a list
of domains that it probably contains and it's list of exons. It also contains a list of missing domains which are
the keys of domains that are in the transcript but aren't in any of the exons.
Each domain in a transcript contains a key and each exon contains a list of domain keys it probably contains.
The columns in the output file are delimited by ";" and the columns are:
GeneID;TranscriptID;Exons;MissingDomains

The exons are delimited by '|' and the data of each exon is delimitted by '@'
The data within each exon is as follows:
start location@stop location@list of domains in the exons

The data within each domain is delimited the same way:
domain key@domain start@domain stop@domain name
the list of domains in an exon is a list of the keys of each domain in the exon and they are delimited by '%'

The missing domains are delimited by '|'

Transcripts from the user data that aren't in the database or are predicted to not contain any domains are
left out.
'''

'''
Before running the program make sure you have the 2 input files in the same folder.
You can change these file names here as well as the output file name
'''
userGTFFileName = "transcripts.gtf"
dataFileName = "mrna_peptide.txt"

outputFileName = "transcriptsWdomains.txt"

delimiter1 = ";"
delimiter2 = "|"
delimiter3 = "@"
delimiter4 = "%"

def parseGTF(filename):
    with open(filename) as f:
        content = f.readlines()
    userTranscripts = []
    transcript = []
    transcriptIDprev = ''
    geneIDprev = ''
    relativeStop = 0
    cds = []
    '''
    The relative start and stop values are relative to the finished protein.
    Here there are 2 cases. 1: if the read transcript is the same as the previous one so it need to be added to the
     same entry and the relative start and stop area need to be adjusted acording to the previous exon.
     2: the transcript is different than the previous one so everything must start again.
    '''
    for line in content:
        stline = line.split("\t")
        if stline[2] == 'exon':
            geneID = stline[8].split("\"")[1]
            transcriptID = stline[8].split("\"")[3]
            cdsStart = stline[3]
            cdsStop = stline[4]
            cdsLength = int(cdsStop) - int(cdsStart)
            # could do abs(cdsLength) +1
            if cdsLength > 0:
                cdsLength = cdsLength +1
            else:
                cdsLength = (-1)*cdsLength +1

            # same transcript
            if transcriptIDprev == transcriptID:
                relativeStart = relativeStop +1
                relativeStop = relativeStart + cdsLength
                exon = []
                # cds start,end,length, and where is it on the transcript
                exon.extend((cdsStart, cdsStop, cdsLength, relativeStart, relativeStop))
                cds.append(exon)
            else:
                relativeStart = 1
                relativeStop = relativeStart + cdsLength
                transcript.extend((geneIDprev, transcriptIDprev, cds))
                userTranscripts.append(transcript)
                cds = []
                exon = []
                transcript = []
                exon.extend((cdsStart, cdsStop, cdsLength, relativeStart, relativeStop))
                cds.append(exon)
            transcriptIDprev = transcriptID
            geneIDprev = geneID
    #print(userTranscripts)
    userTranscripts.pop(0)
    transcript.extend((geneIDprev, transcriptIDprev, cds))
    userTranscripts.append(transcript)
    return userTranscripts



def loadData(datafile):
    with open(datafile) as f:
        content = f.readlines()
        content.pop(0)
        data = []
    for line in content:
        stline = line.split(";")
        domainsT = stline[6].split("|")
        domains = []
        num = 1
        for doms in domainsT:
            stline2 = doms.split("@")
            stline2.insert(0, num)
            domains.append(stline2)
            num += 1
        domainsT = stline[7].split("|")
        for doms in domainsT:
            stline2 = doms.split("@")
            stline2.insert(0, num)
            domains.append(stline2)
            num += 1
        exons = []
        eStart = stline[17].split("|")
        eStop = stline[18].split("|")
        eStart.pop()
        eStop.pop()
        for (starts, stops) in zip(eStart, eStop):
            stline2 = []
            stline2.extend((starts, stops))
            exons.append(stline2)

        dataLine = []
        if domains[0][1] != '' or domains[1][1] != '':
            dataLine.extend((stline[3], stline[1], stline[19][:-2], domains, exons))
            data.append(dataLine)
    print(data[:5])
    return data

# needs to be done once for every upgrade of db
# after one use should add to db the new data
def assignDomainsToExons():
    # built from
    # 0.symbol
    # 1.np_1131311
    # 2.id
    # 3.[domains]
    # 4.[exon loc pairs]
    for transcript in data:
        relativeStart = 1
        relativeStop = 0
        for exon in transcript[4]:
            if exon[0] != '' and exon[1] != '':
                exonLength = int(exon[1])-int(exon[0])
                relativeStop = relativeStart + exonLength
                domainList = []
                for domain in transcript[3]:
                    if domain[1].isdigit():
                        domainNum = domain[0]
                        domainStart = int(domain[1])*3 -2
                        domainStop = int(domain[2])*3
                        # if domain start inside relative
                        # or if domain end inside relative
                        # add domain number to domainList of the exon
                        if (relativeStart <= domainStart and domainStart <= relativeStop) or\
                              (relativeStart <= domainStop and domainStop <= relativeStop):
                            domainList.append(domainNum)
                # add the domain list to the exon data
                exon.append(domainList)
                relativeStart = relativeStop + 1

def assignDomainsToUExon(uTranscript):
    uTranscriptName = uTranscript[1]
    newUTranscript = []
    for transcript in data:
        transcriptName = transcript[2]
        # if the transcript have the same id
        # terrible way of finding matches
        # should be done in sql query for the transcript id
        if uTranscriptName == transcriptName:
            uExons = uTranscript[2]
            exons = transcript[4]
            for uExon in uExons:
                uExonStart = uExon[0]
                uExonStop = uExon[1]
                for exon in exons:
                    if len(exon) == 3:
                        exonStart = exon[0]
                        exonStop = exon[1]
                        exonDomains = exon[2]
                        # if the exon from user transcript contains all of the exon from the db
                        if uExonStart <= exonStart and exonStop <= uExonStop:
                            # tell the user exon that is contains the domains of the db exon
                            uExon.append(exonDomains)
            # add a new tuple of the user transcript data with new exonDomains information
            # to a new list
            newUTranscript.extend((uTranscript[0], uTranscript[1], transcript[3], uTranscript[2]))
            break
    return newUTranscript

def makeOutputFile(fileName):
    with open(fileName, "w") as f:
        for uTranscript in userTranscripts:
            # get user exons data for every transcript
            newUTranscript = assignDomainsToUExon(uTranscript)
            if newUTranscript != []:
                uTranscript = newUTranscript
                finalLine = uTranscript[0] + delimiter1 + uTranscript[1] + delimiter1
                domainsLine = ""
                domainsList = []
                for domain in uTranscript[2]:
                    if domain[1].isdigit():
                        domainsLine = domainsLine + str(domain[0]) + delimiter3 + str(domain[1]) + delimiter3 +\
                                      str(domain[2]) + delimiter3 + domain[3] + delimiter2
                        domainNum = []
                        domainNum.extend((domain[0], 0))
                        domainsList.append(domainNum)
                domainsLine = domainsLine[:-1]
                finalLine = finalLine + domainsLine + delimiter1
                exonsLine = ""
                for exon in uTranscript[3]:
                    exonsLine = exonsLine + str(exon[0]) + delimiter3 + str(exon[1])
                    if len(exon) == 6:
                        exonDomains = ""
                        for domain in exon[5]:
                            exonDomains = exonDomains + str(domain) + delimiter4
                            for edomain in domainsList:
                                if edomain[0] == domain:
                                    edomain[1] += 1
                        exonDomains = exonDomains[:-1]
                    exonsLine = exonsLine + delimiter3 + exonDomains + delimiter2
                exonsLine = exonsLine[:-1]
                missingDomains = ""
                for edomain in domainsList:
                    if edomain[1] == 0:
                        missingDomains = missingDomains + str(edomain[0]) +delimiter2

                missingDomains = missingDomains[:-1]
                finalLine = finalLine + exonsLine + delimiter1 + missingDomains
                f.write(finalLine + "\n")
            else:
                uTranscript.append("Transcript not in Database")

data = loadData(dataFileName)
userTranscripts = parseGTF(userGTFFileName)
assignDomainsToExons()
makeOutputFile(outputFileName)




import os
import ftplib
import requests
import progressbar
from sh import gunzip

# add species here to have correct placement in db folders
species = {
    'Mouse':'Mus_musculus',
    'Human':'Homo_sapiens'
}

url = "http://genome-euro.ucsc.edu/cgi-bin/hgTables"

params2 = {
    'hgsid':'605922159_gOUJ14QP9JLMQLftQOx39ehhfaNQ',
    'jsh_pageVertPos': '0',
    'clade': 'mammal',
    'org': 'Mouse',
    'db': 'mm10',
    'hgta_group': 'genes',
    'hgta_track': 'knownGene',
    'hgta_table': 'knownGene',
    'hgta_regionType': 'genome',
    'position': 'chr9:21802635-21865969',
    'hgta_outputType': 'primaryTable',
    'boolshad.sendToGalaxy': '0',
    'boolshad.sendToGreat': '0',
    'boolshad.sendToGenomeSpace': '0',
    'hgta_outFileName': '',
    'hgta_compressType': 'none',
    'hgta_doTopSubmit': 'get output'
}

params = {
    'hgsid':'605984071_MCAGID7KmBvssx6IJozA1Ou1XsSi',
    'jsh_pageVertPos': '0',
    'clade': 'mammal',
    'org': 'Mouse',
    'db': 'mm10',
    'hgta_group': 'genes',
    'hgta_track': 'knownGene',
    'hgta_table': 'kgAlias',
    'hgta_regionType': 'genome',
    'position': 'chr9:21802635-21865969',
    'hgta_outputType': 'primaryTable',
    'boolshad.sendToGalaxy': '0',
    'boolshad.sendToGreat': '0',
    'boolshad.sendToGenomeSpace': '0',
    'hgta_outFileName': '',
    'hgta_compressType': 'none',
    'hgta_doTopSubmit': 'get output'
}
# currently not in use
humans_aliases = {
        'hgsid':'609792333_1QkTn4aGpxMwQjFdN9zh5F7zwJ1F',
        'jsh_pageVertPos':'0',
        'clade':'mammal',
        'org':'Human',
        'db':'hg38',
        'hgta_group':'genes',
        'hgta_track':'knownGene',
        'hgta_table':'kgAlias',
        'position':'chr1:3A11102837-11267747',
        'hgta_regionType':'genome',
        'hgta_outputType':'primaryTable',
        'boolshad.sendToGalaxy':'0',
        'boolshad.sendToGreat':'0',
        'boolshad.sendToGenomeSpace':'0',
        'hgta_outFileName':'',
        'hgta_compressType':'none',
        'hgta_doTopSubmit':'get output'
}
# currently not in use
humans_gene_table = {
        'hgsid':'609935507_iaCZKqxaqa997kPPi25VX3BSJHO2',
        'jsh_pageVertPos':'0',
        'clade':'mammal',
        'org':'Human',
        'db':'hg38',
        'hgta_group':'genes',
        'hgta_track':'knownGene',
        'hgta_table':'knownGene',
        'hgta_regionType':'genome',
        'position':'chr1%3A11102837-11267747',
        'hgta_outputType':'primaryTable',
        'boolshad.sendToGalaxy':'0',
        'boolshad.sendToGreat':'0',
        'boolshad.sendToGenomeSpace':'0',
        'hgta_outFileName':'test.txt',
        'hgta_compressType':'none',
        'hgta_doTopSubmit':'get output'
}
# ftp addresses for human data from ucsc
ftp_address = 'hgdownload.soe.ucsc.edu'
human_aliases = 'goldenPath/hg38/database/kgAlias.txt.gz'
human_knownGene = 'goldenPath/hg38/database/knownGene.txt.gz'

# prompt for user to ask if the user wants to download new tables
table_prompt = 'Download ucsc {} table for {}? (y/N): '
skipping_prompt = 'Skipping {} table for {}'
ask_me_every_time = False
def get_transcript_data():
    raw_data = []
    for param in [params2]:
        if ask_me_every_time:
            user_input = input(table_prompt.format(param['hgta_table'],param['org']))
            if user_input.lower() != 'y':
                print (skipping_prompt.format(param['hgta_table'],param['org']))
                continue
        session = requests.Session()
        print ("Downloading {} table for {} genome...".format(param['hgta_table'],param['org']))
        response = session.post(url, data=param)
        raw_data.append((str(response.content),param))
    return raw_data


def get_transcript_aliases():
    raw_data = []
    for param in [params]:
        if ask_me_every_time:
            user_input = input(table_prompt.format(param['hgta_table'],param['org']))
            if user_input.lower() != 'y':
                print (skipping_prompt.format(param['hgta_table'],param['org']))
                continue
        session = requests.Session()
        print ("Downloading {} table for {} genome...".format(param['hgta_table'],param['org']))
        response = session.post(url,data = params)
        raw_data.append((str(response.content),param))
    return raw_data

ftp_prompt = 'Download {}? (y/N): '
ftp_skipping_prompt = 'Skipping {}'
def download_ftp_data(address,username,password,files):
    print('connecting to: ',address,'...')
    ftp = ftplib.FTP(address)
    print('logging in...')
    ftp.login(username,password)
    for file in files:
        os.makedirs(os.path.dirname(file[1]),exist_ok=True)
        if ask_me_every_time:
            user_input = input(ftp_prompt.format(file[0]))
            if user_input.lower() != 'y':
                print (ftp_skipping_prompt.format(file[0]))
                continue
        print('downloading: ',file[0],'...')
        ftp.sendcmd("TYPE i")
        size = ftp.size(file[0])
        p_bar = progressbar.AnimatedProgressBar(end=size,width=10)
        with open(file[1]+'.gz','wb') as f:
            def callback(chunk):
                f.write(chunk)
                p_bar + len(chunk)
                p_bar.show_progress()
            ftp.retrbinary("RETR " + file[0], callback)
            p_bar + size
            p_bar.show_progress()
        print()
        print('extracting...')
        gunzip(file[1]+'.gz','-f')
        # add \ to \t because backward compatability is important
        with open(file[1], 'r') as f:
            content = f.read()
        content.replace('\t','\\t')
        with open(file[1], 'w') as f:
            f.write(content)
        print('done')

def data_splitter(raw_data):
    lines = raw_data.split('\\n')
    # remove first and last elements, as they are not needed
    del lines[0]
    del lines[-1]
    lines = list(map(lambda x: x + '\n',lines))
    return lines



def write_to_file(data,path):
    os.makedirs(os.path.dirname(path),exist_ok=True)
    print ("Writing {}...".format(path))
    with open(path, 'w') as f:
        f.writelines(data)

def get_specie_name(specie):
    return species[specie]

def main():
    user_input = input('request permission for every download/update? (y/N): ')
    global ask_me_every_time
    ask_me_every_time = (user_input.lower() == 'y')
    if ask_me_every_time:
        print('please choose when needed')
    transcript_data = get_transcript_data()
    transcript_aliases = get_transcript_aliases()
    for alias in transcript_aliases:
        name = 'db/'+get_specie_name(alias[1]['org'])+'/'+alias[1]['hgta_table']+'.txt'
        print ('alias name: ',name)
        write_to_file(data_splitter(alias[0]),name)
    for data in transcript_data:
        name = 'db/'+get_specie_name(data[1]['org'])+'/'+data[1]['hgta_table']+'.txt'
        print ('data name: ',name)
        write_to_file(data_splitter(data[0]),name)

    ftp_files =[(human_aliases,'db/'+species['Human']+'/kgAlias.txt'),(human_knownGene,'db/'+species['Human']+'/knownGene.txt')]
    download_ftp_data(ftp_address,'anonymous','example@post.bgu.ac.il',ftp_files)

if __name__ == '__main__':
    main()




# supported species names
species = ['Homo_sapiens','Mus_musculus']
# converting Names to database locations
databases = {
    'Human':'db/Homo_sapiens.db',
    'Homo_sapiens':'db/Homo_sapiens.db',
    'Mouse': 'db/Mus_musculus.db',
    'Mus_musculus': 'db/Mus_musculus.db'
}



import os
import json
import visualizer
import progressbar
import domains_to_exons
import uuid
import conf

domains_db = 'db/domains.db'

# parse json file into dictionary
# input:
# filename: path to the output file of interface.py - must be json file.
# output:
# data: dictionary of the json data in the file.
def parse_output_file(filename):
    with open(filename,'r') as f:
        json_dump = f.read()
        data = parse_output(json_dump)
    return data

# parse json string into dictionary
# input:
# json_dump: string of json data 
# output:
# transcripts_data: the parsed json string data.
def parse_output(json_dump):
    transcripts_data = {}
    transcripts_data = json.loads(json_dump)
    return transcripts_data

# load all the exons transcript data from the given transcripts names in the user_data
# input:
# user_data: dictionary created from the output file by parse_output_file
# specie: name of the specie from which to take the transcripts
def load_db_data(user_data,specie):
    print ("loading db exons...")
    bar = progressbar.AnimatedProgressBar(end=len(user_data),width=10)
    bar +=1
    bar.show_progress()
    for key,item in user_data.items():
        #doms = domains_to_exons.get_domains(key)
        exons = domains_to_exons.get_exons_by_transcript_id(key,specie)
        item.append(exons)
        bar+=1
        bar.show_progress()
    print("\ndone")

# parses the given output_file and create svgs from it.
# input:
# output_file: path to the output file from interface.py
# specie: specie to visualize
# output:
# target_folder: path to the folder containing the new svgs
def load_and_visualize(output_file,specie):
    data = parse_output_file(output_file)
    load_db_data(data,specie)
    target_folder = str(specie) + '/' + str(uuid.uuid4())
    print("creating svgs...")
    bar = progressbar.AnimatedProgressBar(end=len(data),width=10)
    bar+=1
    bar.show_progress()
    for index, (key,item) in enumerate(data.items()):
        bar+=1
        bar.show_progress()
        visualizer.visualize_transcript(target_folder,item)
    print ('\ndone.')
    return target_folder


def main(output_file,specie):
    folder = load_and_visualize(output_file,specie)

if __name__ == '__main__':
    main('mouse_output', 'Mus_musculus')




import gbk_parser

records= gbk_parser.records

def write_comb():
    with open('comb.txt','w') as f:
        combs = []
        for record in records:
            for feature in record.features:
                f.write(str(feature.type))
                f.write(str(feature.location)+"\n")
write_comb()



import sys
import time

class ProgressBar(object):
    """ProgressBar class holds the options of the progress bar.
    The options are:
        start   State from which start the progress. For example, if start is 
                5 and the end is 10, the progress of this state is 50%
        end     State in which the progress has terminated.
        width   --
        fill    String to use for "filled" used to represent the progress
        blank   String to use for "filled" used to represent remaining space.
        format  Format
        incremental
    """
    def __init__(self, start=0, end=10, width=12, fill='=', blank='.', format='[%(fill)s>%(blank)s] %(progress)s%%', incremental=True):
        super(ProgressBar, self).__init__()

        self.start = start
        self.end = end
        self.width = width
        self.fill = fill
        self.blank = blank
        self.format = format
        self.incremental = incremental
        self.step = 100 / float(width) #fix
        self.reset()

    def __add__(self, increment):
        increment = self._get_progress(increment)
        if 100 > self.progress + increment:
            self.progress += increment
        else:
            self.progress = 100
        return self

    def __str__(self):
        progressed = int(self.progress / self.step) #fix
        fill = progressed * self.fill
        blank = (self.width - progressed) * self.blank
        return self.format % {'fill': fill, 'blank': blank, 'progress': int(self.progress)}

    __repr__ = __str__

    def _get_progress(self, increment):
        return float(increment * 100) / self.end

    def reset(self):
        """Resets the current progress to the start point"""
        self.progress = self._get_progress(self.start)
        return self


class AnimatedProgressBar(ProgressBar):
    """Extends ProgressBar to allow you to use it straighforward on a script.
    Accepts an extra keyword argument named `stdout` (by default use sys.stdout)
    and may be any file-object to which send the progress status.
    """
    def __init__(self, *args, **kwargs):
        super(AnimatedProgressBar, self).__init__(*args, **kwargs)
        self.stdout = kwargs.get('stdout', sys.stdout)

    def show_progress(self):
        if hasattr(self.stdout, 'isatty') and self.stdout.isatty():
            self.stdout.write('\r')
        else:
            self.stdout.write('\n')
        self.stdout.write(str(self))
        self.stdout.flush()


if __name__ == '__main__':
    p = AnimatedProgressBar(end=100, width=80)

    while True:
        p + 5
        p.show_progress()
        time.sleep(0.1)
        if p.progress == 100:
            break
    print ()#new line




import os
import threading
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation
from Bio import SeqIO
import progressbar
import conf
from functools import partial
# get all sequence records from the gbk file
gbk_file = "db/{}/protein.gbk"
def get_records():
    records = {specie:parse_proteins(specie) for specie in conf.species}
    return records

#records = [record for record in SeqIO.parse(gbk_file,"genbank")]
def parse_proteins(specie):
    print ("parsing protein.gbk of {}".format(specie))
    records=[]
    # length as of 2016 of mouse
    length = 76216
    p_bar = progressbar.AnimatedProgressBar(end=length,width=10)
    p_bar+1
    for record in SeqIO.parse(gbk_file.format(specie),"genbank"):
        records.append(record)
        p_bar+1
        p_bar.show_progress()
    print()
    print("done")
    return records

def write_annotations(specie):
    print ("writing annotation.txt")
    with open("annotation.txt","w") as annotations_file:
        for record in records[specie]:
            annotations_file.write(str(record))


def write_cds(specie):
    print ("writing cds.txt")
    with open("cds.txt","w") as cds_file:
        for record in records[specie]:
            features = [feature for feature in record.features if feature.type == "CDS"]
            for feature in features:
                cds_file.write(str(feature))

def write_sites(specie):
    print("writing sites.txt")
    with open("sites.txt","w") as sites_file:
        for record in records[specie]:
            features = [feature for feature in record.features if feature.type== "Site"]
            for feature in features:
                sites_file.write(str(feature))
                # can take begin@end@name maybe

def write_regions(specie):
    print("writing regions.txt")
    with open("regions.txt","w") as regions_file:
        for record in records[specie]:
            features = [feature for feature in record.features if feature.type == "Region"]
            for feature in features:
                regions_file.write(str(feature))

def write_aliases(specie):
    print("writing aliases.txt")
    with open("aliases.txt","w") as aliases_file:
        aliases = set()
        for record in records[specie]:
            features = [feature for feature in record.features if feature.type == "CDS"]
            for feature in features:
                gene_aliases =""
                gene_name =""
                if "gene" in feature.qualifiers:
                    gene_name = feature.qualifiers["gene"][0]
                if "gene_synonym" in feature.qualifiers:
                    gene_aliases = feature.qualifiers["gene_synonym"][0]
                if gene_name:
                    aliases.add(gene_name + ";" + gene_aliases)
        for aliase in aliases:
            aliases_file.write(aliase +"\n")
if __name__ == "__main__":
    for specie in conf.species:
        func = partial(write_aliases,specie)
        t = threading.Thread(target=func)
        t.start()



import os.path
import sys
import shutil
import subprocess
print("Updating ucsc tables...")
exec(open('ucsc_downloader.py').read())
exec(open('updater.py').read())
if (os.path.exists('db/readme_new')):
    shutil.move('db/readme_new','db/readme_old')
print('recreating the database...')
exec(open('db_creator.py').read())
print('assinging domains to exons, this will take a while.')
exec(open('domains_to_exons.py').read())



import os
import sqlite3
import getopt
import sys
import conf

def parseGTF(filename):
    with open(filename) as f:
        content = f.readlines()
    userTranscripts = []
    transcript = []
    transcriptIDprev = ''
    geneIDprev = ''
    relativeStop = 0
    cds = []
    '''
    The relative start and stop values are relative to the finished protein.
    Here there are 2 cases. 1: if the read transcript is the same as the previous one so it need to be added to the
     same entry and the relative start and stop area need to be adjusted acording to the previous exon.
     2: the transcript is different than the previous one so everything must start again.
    '''
    for line in content:
        stline = line.split("\t")
        if stline[2] == 'exon':
            geneID = stline[8].split("\"")[1]
            transcriptID = stline[8].split("\"")[3]
            cdsStart = stline[3]
            cdsStop = stline[4]
            cdsLength = int(cdsStop) - int(cdsStart)
            # could do abs(cdsLength) +1
            if cdsLength > 0:
                cdsLength = cdsLength +1
            else:
                cdsLength = (-1)*cdsLength +1

            # same transcript
            if transcriptIDprev == transcriptID:
                relativeStart = relativeStop +1
                relativeStop = relativeStart + cdsLength
                exon = []
                # cds start,end,length, and where is it on the transcript
                exon.extend((cdsStart, cdsStop, cdsLength, relativeStart, relativeStop))
                cds.append(exon)
            else:
                relativeStart = 1
                relativeStop = relativeStart + cdsLength
                transcript.extend((geneIDprev, transcriptIDprev, cds))
                userTranscripts.append(transcript)
                cds = []
                exon = []
                transcript = []
                exon.extend((cdsStart, cdsStop, cdsLength, relativeStart, relativeStop))
                cds.append(exon)
            transcriptIDprev = transcriptID
            geneIDprev = geneID
    #print(userTranscripts)
    userTranscripts.pop(0)
    transcript.extend((geneIDprev, transcriptIDprev, cds))
    userTranscripts.append(transcript)
    return userTranscripts

help_message = "parser.py -i <inputfile> -o <outputfile> -s specie"

def main(argv):
    input_file = ''
    out_putfile =''
    specie = ''
    try:
        opts,args = getopt.getopt(argv,'h:i:o:s')
    except getopt.GetoptError:
        print(help_message)
        sys.exit(2)
    for opt,arg in opts:
        if opt == '-h':
            print(help_message)
            sys.exit()
        elif opt in ('-i','-ifile'):
           input_file = arg
        elif opt in('-o','-ofile'):
            output_file = arg
        elif opt in ('-s','-specie'):
            specie = arg
    if input_file =='' or specie = '':
        print(help_message)
        sys.exit()
    data = loadData('db/comb.db')
    for d in data[:5]:
        print (str(d) +'\n')
    #parsed_gtf =  parseGTF(input_file)
    #print (parsed_gtf[:10])
    #for transcript in parsed_gtf[:10]:
    #    print (assignDomainsToUExon(transcript))

def assignDomainsToExons(data):
    for transcript in data:
        relativeStart = 1
        relativeStop = 0
        for exon in transcript[4]:
            if exon[0] != '' and exon[1] != '':
                exonLength = int(exon[1])-int(exon[0])
                relativeStop = relativeStart + exonLength
                domainList = []
                for domain in transcript[3]:
                    if domain[1].isdigit():
                        domainNum = domain[0]
                        domainStart = int(domain[1])*3 -2
                        domainStop = int(domain[2])*3
                        if (relativeStart <= domainStart and domainStart <= relativeStop) or\
                              (relativeStart <= domainStop and domainStop <= relativeStop):
                            domainList.append(domainNum)
                exon.append(domainList)
                relativeStart = relativeStop + 1

# for a given transcript from gtf file, assign domains to exons
# call for every transcript of user
def assignDomainsToUExon(data,uTranscript):
    uTranscriptName = uTranscript[1]
    newUTranscript = []
    for transcript in data:
        transcriptName = transcript[2]
        if uTranscriptName == transcriptName:
            uExons = uTranscript[2]
            exons = transcript[4]
            for uExon in uExons:
                uExonStart = uExon[0]
                uExonStop = uExon[1]
                for exon in exons:
                    if len(exon) == 3:
                        exonStart = exon[0]
                        exonStop = exon[1]
                        exonDomains = exon[2]
                        if uExonStart <= exonStart and exonStop <= uExonStop:
                            uExon.append(exonDomains)
            newUTranscript.extend((uTranscript[0], uTranscript[1], transcript[3], uTranscript[2]))
            break
    return newUTranscript


def loadData(db_path,specie):
    with sqlite3.connect(conf.databases[specie]) as con:
        data =[]
        con.row_factory = sqlite3.Row
        cursor = con.cursor()
        result = cursor.execute("SELECT * FROM genebank ")
        while True:
            row = result.fetchone()
            if not row:
                    break
            # get the symbol
            symbol = row['symbol']
            coded_by= row['coded_by'].split(':')[0]
            # missing USCS transcript id
            transcript_id = ''
            sites = []
            regions = []
            sites_text = row['sites'].split(',')
            for s in sites_text:
                if s == '':
                    continue
                site = {}
                splitted = s.split('[')
                site['name'] = splitted[0]
                site['start'] = splitted[1].split(':')[0]
                site['end'] =splitted[1].split(':')[1].split(']')[0]
                sites.append(site)
            regions_text = row['regions'].split(',')
            for r in regions_text:
                if s == '':
                    continue
                region = {}
                splitted = s.split('[')
                region['name'] = splitted[0]
                region['start'] = splitted[1].split(':')[0]
                region['end'] =splitted[1].split(':')[1].split(']')[0]
                regions.append(region)
            # cds start and ends are missing (big numbers)
            # they are the exons start and end
            cds_numbers = []
            data.append((symbol,coded_by,transcript_id,sites,regions, cds_numbers))
        return data

if __name__ == '__main__':
    main(sys.argv[1:])



import os
import json
import time
import dill
import sqlite3 as lite
import progressbar
from pathos.multiprocessing import ProcessingPool, ThreadingPool
from functools import partial
import conf

domains_db = 'db/domains.db'
alias_db = 'db/aliases.db'
comb_db = 'db/comb.db'
transcripts_db = 'db/transcript_data.db'
num_threads = 1
bar = 0
# run on all transcripts in transcripts.db
# for each one find matching alias in alias.db
# use alias to find sites and regions data in comb.db
# get all domains of a transcript
#TODO DOESNT WORK
def get_domains(transcript_id, specie):
    # use better aliases - obselete
    #alias_db = 'db/better_aliases.db'
    with lite.connect(conf.databases[specie]) as con:
        cursor = con.cursor()
        cursor.execute("SELECT name from aliases WHERE transcript_id = ?",(transcript_id,))
        #cursor.execute("SELECT aliases from genes WHERE transcript_id = ?",(transcript_id,))
        data = cursor.fetchall()
        if data:
            # create a list of the aliases
            aliases = list(set([tup[0] for tup in data]))
            #aliases = data[0].split(';')
        else:
            return None
    with lite.connect(conf.databases[specie]) as con:
        for alias in aliases:
            cursor = con.cursor()
            cursor.execute("SELECT sites,regions from genebank WHERE symbol = ?",(alias,))
            results = cursor.fetchall()
            domain_list = []
            #print('result for {} alias {} is:\n{}'.format(transcript_id,alias,results))
            if results:
                # if counter is 0, domain type is site, if counter is 1, domain type is region
                domain_types = ['site','region']
                for counter,domains in enumerate(results[0]):
                    splitted = domains.split(',')
                    #print ('splitted: ',splitted)
                    for result in splitted:
                        if '[' not in result or ']' not in result:
                            #print('bad result for {} alias {} result:\n'.format(transcript_id,alias,result))
                            continue
                        modified = result.replace(',',':')
                        part_one = '['.join(modified.split('[')[:-1])
                        part_two = modified.split('[')[-1].replace(']',':').split(':')
                        domain = {}
                        # check that line is not empty
                        #print ('modified: ',modified)
                        if not modified[0]:
                            #print('bad modified for {} alias {} modified:\n'.format(transcript_id,alias,modified))
                            continue
                        domain['type'] = domain_types[counter]
                        domain['name'] = part_one
                        if len(part_two) < 2:
                            print ('fucked up on part_two')
                            print ('result:', result)
                            print ('part_one: ',part_one)
                            print ('part_two: ',part_two)
                            print ('modified: ',modified)

                        domain['start'] = part_two[0]
                        domain['end'] = part_two[1]
                        domain['index'] = len(domain_list)
                        #print ('domain: ', domain)
                        if not str.isdigit(domain['start']) or not str.isdigit(domain['end']):
                            #print('bad strings in domain for {} alias {} domain:\n'.format(transcript_id,alias,domain))
                            continue
                        domain_list.append(domain)
                #print('found for {} alias {} doms list:\n{}'.format(transcript_id,alias,domain_list))
                return domain_list
            else:
                pass
                #print('found no sites or regions for {} alias {}'.format(transcript_id,alias))

def get_exons_by_transcript_id(transcript_id,specie):
    with lite.connect(conf.databases[specie]) as con:
        con.row_factory = lite.Row
        cursor = con.cursor()
        cursor.execute("SELECT * FROM transcripts WHERE name = ?",(transcript_id,))
        # currently use the first result only (there shouldnt be more then one)
        result = cursor.fetchone()
    exons = get_exons(result)
    return exons

def get_exons(result):
    if not result:
        #print('nothing in result')
        return
    exons =[]
    starts = result['exon_starts'].split(',')
    ends = result['exon_ends'].split(',')
    first_exon_start = int(starts[0])
    last_end = 0
    for i in range(int(result['exon_count'])):
        exon = {}
        exon['index'] = i
        exon['start'] = int(starts[i])
        exon['end'] = int(ends[i])
        exon['length'] = abs(exon['end'] - exon['start'])
        # the exon relative position
        exon['relative_start'] = last_end + 1
        exon['relative_end'] = last_end + 1 + exon['length']
        #exon['relative_start'] = abs(exon['start'] - first_exon_start) + 1
        #exon['relative_end'] = exon['relative_start'] + exon['length']
        last_end = exon['relative_end']
        exon['transcript_id'] = result['name']
        exons.append(exon)
    return exons


def assignDomainsToExons(transcript_id, domains, specie):
    # get data about the transcript
    with lite.connect(conf.databases[specie]) as con:
        con.row_factory = lite.Row
        cursor = con.cursor()
        cursor.execute("SELECT * FROM transcripts WHERE name = ?",(transcript_id,))
        # currently use the first result only (there shouldnt be more then one)
        result = cursor.fetchone()
    exons = get_exons(result)
    #print("exons",exons)
    #print("domains",domains)
    # TODO relative should be counted without spaces between exons
    # next statement might not be accurate
    # i dont know from what each domain relative location is relative to
    relative_start = 1
    relative_stop = 0
    last_exon = None
    for exon in exons:
        # domains indexes will be used as strings
        # states will be start,end,contains,contained
        exon['domains_states'] = {}
        relative_stop = relative_start + exon['length']
        domains_in_exon = []
        if domains == None:
            return
        for domain in domains:
            if (not str.isdigit(domain['start'])):
                print ("FAILED ON {} domain is: {}".format(transcript_id,domain))
                sys.exit(2)
            dom_start = int(domain['start']) * 3 - 2
            dom_stop = int(domain['end']) * 3

            # create ranges of numbers that represents the domain and exon ranges
            if dom_start < dom_stop:
                dom_range = range(dom_start,dom_stop+1)
            else:
                dom_range = range(dom_stop,dom_start+1)

            exon_range = set(range(relative_start,relative_stop+1))
            intersection = exon_range.intersection(dom_range)

            dom_start_in_exon = False
            dom_end_in_exon = False

            if not intersection:
                # empty, no overlap
                continue

            if dom_start in intersection:
                # exon start location in domain
                # domain is atleast partialy on exon
                dom_start_in_exon = True

            if dom_stop in intersection:
                # exon end location in domain
                # domain is atleast partialy on exon
                dom_end_in_exon = True

            dom_index = str(domain['index'])
            if dom_end_in_exon and dom_start_in_exon:
                # domain fully in exon
                exon['domains_states'][dom_index] = 'contained'
                domains_in_exon.append(domain)
                continue
            if dom_start_in_exon:
                exon['domains_states'][dom_index] = 'start'
                domains_in_exon.append(domain)
                continue
            if dom_end_in_exon:
                exon['domains_states'][dom_index] = 'end'
                domains_in_exon.append(domain)
                continue
            # there is an intersection but dom_start and dom_end not in it
            # that means that the domain contains the exon
            exon['domains_states'][dom_index] = 'contains'
            domains_in_exon.append(domain)
            continue

            #dom_start_in = relative_start <= dom_start and dom_start <= relative_stop
            #dom_end_in = relative_start <= dom_stop and dom_stop <= relative_stop

            #if dom_start_in or dom_end_in:
            #    domains_in_exon.append(domain)
        nums = [domain['index'] for domain in domains_in_exon]
        exon['domains'] = domains_in_exon
        relative_start_mod = 0
        if last_exon:
            relative_start_mod = abs(exon['start'] - last_exon['end'])
        relative_start = relative_stop + 1 + relative_start_mod
        last_exon = exon
    return exons

def get_bar():
    return bar

def assign_and_get(specie,name):
    #print ('assign and get specie {} name {}'.format(specie,name))
    global bar
    bar = bar+1
    if name:
        return assignDomainsToExons(name,get_domains(name,specie),specie)
    return None

def write_to_db(data,specie):
    # write all domains in exons to db
    # data is build [(id,index,[domains_nums]),...]
    #print('writing to db/domains.db...')
    with lite.connect(conf.databases[specie]) as con:
        cursor = con.cursor()
        cursor.executescript("drop table if exists domains;")
        cursor.execute("CREATE TABLE domains(transcript_id TEXT, exon_index INT,domains_states TEXT, domains_list TEXT)")
        cursor.executemany("INSERT INTO domains VALUES(?,?,?,?)",data)



def main(specie):
    #print(get_domains('uc007aeu.1'))
    #print(get_domains('uc012gqd.1'))
    #print(get_domains('uc007afi.2'))
    print("loading {} data...".format(specie))
    with lite.connect(conf.databases[specie]) as con:
        cursor = con.cursor()
        # TODO might be a problem here
        cursor.execute("SELECT DISTINCT transcript_id from aliases")
        result = cursor.fetchall()
    names = [value[0] for value in result]
    print("creating transcript database for {}".format(specie))
    # give this thing a progress bar
    global bar
    bar = progressbar.AnimatedProgressBar(end=len(names),width=10)
    pool = ThreadingPool(num_threads)
    assign_and_get_with_specie = partial(assign_and_get, specie)
    result = pool.amap(assign_and_get_with_specie,names)
    while True:
        if result.ready():
            break
        bar.show_progress()
        time.sleep(1)
    data = list(result.get())
    # dark magic incoming
    # flatten the list
    # make it a list of tuples of id,index,domainlist
    data = [(exon['transcript_id'],exon['index'],json.dumps((exon['domains_states'])),json.dumps(exon['domains'])) for exons in data if exons != None for exon in exons if exon != None]
    #ids = [tup[0] for tup in data]
    #indexes = [tup[1] for tup in data]
    #domains = [tup[2] for tup in data]
    #data = zip(ids,indexes,names)
    write_to_db(data,specie)
    print()
if __name__ == '__main__':
    for specie in conf.species:
        user_input = input("Create the {} domains database? (Y/n): ".format(specie))
        if user_input.lower() == 'n':
            continue
        main(specie)




import os
import conf

knownGene_path = 'db/{}/knownGene.txt'
# parse a given knownGene table from ucsc
# returns a nice dictionary
def parse_knownGene(path):
    # find file
    with open(path,'r') as f:
        lines = f.readlines()
    names = {}
    for line in lines:
        line = line.replace('\n','').replace('\\n','')
        splitted_line = line.split('\\t')
        if len(splitted_line) <2:
            splitted_line = line.split('\t')
        data = {}
        data['name'] = splitted_line[0]
        data['chrom'] = splitted_line[1]
        data['strand'] = splitted_line[2]
        data['tx_start'] = splitted_line[3]
        data['tx_end'] = splitted_line[4]
        data['cds_start'] = splitted_line[5]
        data['cds_end'] = splitted_line[6]
        data['exon_count'] = splitted_line[7]
        data['exon_starts'] = splitted_line[8]
        data['exon_ends'] = splitted_line[9]
        data['protein_id'] = splitted_line[10]
        data['align_id'] = splitted_line[11]
        names[data['name']] = data
    return names

def main():
    for specie in conf.species:
        parse_knownGene(knownGene_path.format(specie))

if __name__ == '__main__':
    main()




from distutils.core import setup
from setuptools import find_packages
import sys
if sys.version_info < (3, 5):
    raise RuntimeError("This package requres Python 3.5+")

setup(
    name='dochap',
    version='1.0.4.1',
    author='Nitzan Elbaz',
    author_email='elbazni@post.bgu.ac.il',
    packages = ['dochap'],
    url='https://github.com/nitzanel/dochap',
    license='MIT',
    description='do some intresting things with your gtf files',
    long_description='some more info',
    zip_safe=False,
    install_requires=['pathos','dill'],
    python_requires='>=3.5'
)



# ADD NEW DATA FROM UCSC, AND FIND WAY TO AUTO DOWNLOAD AND CHECK FOR UPDATE FOR IT
# NEED TABLE OF ALIASES FROM TRANSCRIPT_ID TO GENE_NAME
# NEED TABLE OF TRANSCRIPT DATA LIKE EXONS LOCATIONS, AND CDS LOCATIONS WE NEED
from ftplib import FTP
import tarfile
import sys
import os.path
import subprocess
import progressbar
import pathlib
import conf
from sh import gunzip
prot_path = "genomes/{}/protein/protein.gbk.gz"
filename = "protein.gbk.gz"
# species is Homo_sapiens, Mus_musculus
extract_path = "db/{}/"
readme_file = 'genomes/{}/README_CURRENT_RELEASE'
print ("connecting to ftp.ncbi.nlm.nih.gov...")
ftp = FTP("ftp.ncbi.nlm.nih.gov")
print ("logging in...")
ftp.login()
prompt = 'Update {} data? (y/N): '
skipping_prompt = 'Skipping {}'
for specie in conf.species:
    user_input = input(prompt.format(specie))
    if user_input.lower() != 'y':
        print(skipping_prompt.format(specie))
        continue
    formatted_extract_path = extract_path.format(specie)
    print ("Creating directory {} ".format(formatted_extract_path))
    pathlib.Path(formatted_extract_path).mkdir(parents=True, exist_ok=True)
    print ("downloading {} data".format(specie))
    ftp.sendcmd("TYPE i")
    readme_size = ftp.size(readme_file.format(specie))
    readme_progress = progressbar.AnimatedProgressBar(end=readme_size,width=10)
    if os.path.isfile(formatted_extract_path+ "readme_old"):
        print("readme_old found, checking againts existing database...")
    else:
        print("readme_old not found, downloading new database...")
    print ("downloading readme_file...")
    with open(formatted_extract_path+"readme_new",'w') as f:
        def callback(chunk):
            f.write(chunk)
            readme_progress + len(chunk)
            readme_progress.show_progress()
        ftp.retrlines("RETR " + readme_file.format(specie),callback)
        readme_progress+readme_size
        readme_progress.show_progress()
    # empty line
    print()
    new_readme_lines=[]
    old_readme_lines=[]
    if os.path.isfile(formatted_extract_path+"readme_old"):
        with open(formatted_extract_path+"readme_old","r") as readme:
            old_readme_lines = readme.readlines()
    with open(formatted_extract_path+"readme_new","r") as new_readme:
        new_readme_lines = new_readme.readlines()
    if new_readme_lines == old_readme_lines and os.path.isfile(formatted_extract_path+"aliases.db") and os.path.isfile(formatted_extract_path+"comb.db"):
        print("{} database is up to date.".format(specie))
    elif not os.path.isfile(formatted_extract_path + 'protein.gbk'):
        print ("downloading " + filename + "...")
        ftp.sendcmd("TYPE i")
        archive_size = ftp.size(prot_path.format(specie))
        archive_progress = progressbar.AnimatedProgressBar(end=archive_size,width=10)
        with open(formatted_extract_path+filename,'wb') as f:
            def callback(chunk):
                f.write(chunk)
                archive_progress + len(chunk)
                archive_progress.show_progress()
            ftp.retrbinary("RETR " + prot_path.format(specie),callback)
            archive_progress + archive_size
            archive_progress.show_progress()
            print()
            print ("download complete.")
        gunzip(formatted_extract_path+filename,'-f')
    print ("Moving new readme file...")
    os.rename(formatted_extract_path+"readme_new", formatted_extract_path+"readme_old")
print ("starting database upgrade...")
ftp.quit()
#print ("EXITING NOW, PLEASE REMOVE ME")
#sys.exit(2)
# if the python script calls the updater.sh permission issues may arise
#subprocess.call(['./updater.sh'])



import sqlite3 as lite
import sys
import os
import threading
import gbk_parser
import ucsc_parser
import conf
records = gbk_parser.get_records()

# creates aliases in specie database file.
# must have a kgAlias.txt file in the specie database folder.
# Input:
# specie: string of the specie (must be one from conf.py)
def create_better_aliases_db(specie):
    with open('db/{}/kgAlias.txt'.format(specie),'r') as f:
        aliases_lines = f.readlines()

    values_keys = [tuple(line.replace('\n','').split('\\t')) for line in aliases_lines]
    try:
        zipped = [(key,value) for value,key in values_keys]
    except:
        values_keys = [tuple(line.replace('\n','').split('\t')) for line in aliases_lines]
        zipped = [(key,value) for value,key in values_keys]
    # write pairs to db
    with lite.connect(conf.databases[specie]) as con:
        cursor = con.cursor()
        cursor.executescript("drop table if exists aliases;")
        cursor.execute("CREATE TABLE aliases (name TEXT, transcript_id TEXT)")
        cursor.executemany('INSERT INTO aliases VALUES(?,?)',zipped)

#
def create_transcript_data_db(specie):
    print ("Creating transcript database for {}...".format(specie))
    with lite.connect(conf.databases[specie]) as con:
        names = ucsc_parser.parse_knownGene(ucsc_parser.knownGene_path.format(specie))
        cur = con.cursor()
        cur.executescript("drop table if exists transcripts;")
        cur.execute("CREATE TABLE transcripts(name TEXT,\
                    chrom TEXT,\
                    strand TEXT,\
                    tx_start TEXT,\
                    tx_end TEXT,\
                    cds_start TEXT,\
                    cds_end TEXT,\
                    exon_count TEXT,\
                    exon_starts TEXT,\
                    exon_ends TEXT,\
                    protein_id TEXT,\
                    align_id TEXT)")

        for name in names:
            values = tuple(names[name].values())
            cur.execute("INSERT INTO transcripts VALUES(?,?,?,?,?,?,?,?,?,?,?,?)",values)

def create_comb_db(specie):
    with lite.connect(conf.databases[specie]) as con:
        print("Creating database for {}...".format(specie))
        cur = con.cursor()
        cur.execute("drop table if exists genebank;")
        cur.execute("CREATE TABLE genebank(Id INT, symbol TEXT, db_xref TEXT, coded_by TEXT, chromosome TEXT,strain TEXT, cds TEXT, sites TEXT, regions TEXT)")
        for index,record in enumerate(records[specie]):
            sites=[]
            regions=[]
            cds = []
            location = ""
            name =""
            db_xref=""
            coded_by =""
            chromosome =""
            strain = ""
            for feature in record.features:
                if feature.type == 'source':
                    if 'strain' in feature.qualifiers:
                        strain = feature.qualifiers['strain'][0]
                    if 'chromosome' in feature.qualifiers:
                        chromosome = feature.qualifiers['chromosome'][0]
                if feature.type == 'CDS':
                    cds.append(str(feature))
                    if 'gene' in feature.qualifiers:
                        name = feature.qualifiers['gene'][0]
                    if 'coded_by' in feature.qualifiers:
                        coded_by = feature.qualifiers['coded_by'][0]
                    if 'db_xref' in feature.qualifiers:
                        db_xref = feature.qualifiers['db_xref'][0]
                    location = str(feature.location)
                if feature.type == 'Site':
                    sites.append(feature.qualifiers['site_type'][0]+str(feature.location))
                if feature.type == 'Region':
                    regions.append(feature.qualifiers['region_name'][0]+str(feature.location))
            sites_comb = ','.join(sites)
            region_comb = ','.join(regions)
            cds_comb = ','.join(cds)
            if sites_comb == '' and region_comb == '':
                continue
            cur.execute("INSERT INTO genebank VALUES(?, ?, ?, ?, ?, ?,  ?, ?,?)",(index,name,db_xref,coded_by,chromosome,strain,cds_comb,sites_comb,region_comb))
            index+=1

if __name__ == "__main__":
    for specie in conf.species:
        create_comb_db(specie)
        create_better_aliases_db(specie)
        create_transcript_data_db(specie)
